#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aracnid_utils']

package_data = \
{'': ['*']}

install_requires = \
['aracnid-logger', 'python-dateutil ~=2.8']

setup(name='aracnid_utils',
      version='1.0.0a4',
      description='This package contains utilities that I use frequently.',
      author='Jason Romano',
      author_email='aracnid@gmail.com',
      url='https://github.com/aracnid/aracnid-utils',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='~=3.8',
     )
